%DEMOVERLADEBRUECKERICCATI - Dieses Skript simuliert eine Verladebrcke
% welche mit einem Riccati-Regler geregelt wird. 
%
% 
% Zu spezifierende Parameter:
%    mG         - Masse des Lastegewichts
%    mK         - Masse der Laufkatze
%    L          - Seillnge   
%    Q          - Gewichtungsmatrix der Zustnde
%    R          - Gewichtung der Stellgre
%
% Autor: Martin Pfeifer 
% Bearbeitet von: Jona Maurer 
% Insitut fr Regelung und Steurungssysteme (IRS)
% Karlsruher Institut fr Technologie
% E-mail: martin.pfeifer@kit.edu 
% Website: http://www.irs.kit.edu
% April 2015; Letzte nderung: 26-April-2016

%------------- BEGIN CODE --------------

%% Init
clc
clear all
close all

%% SISO-Modell laden
% Eingangsgre: Kraft auf Laufkatze
% Ausgangsgre: Position der Laufkatze

% Konstanten der Verladebrcke
mG = 1000;    % Masse des Lastgewichts
mK = 500;     % Masse der Laufkatze
L  = 10;      % Seillnge 
g  = 9.81;    % Gravitationskonstante

% linearisiertes Systemmodell fr Riccati-Entwurf
a43 = -(mG+mK)*g/(L*mK);
a23 = mG*g/mK;

% Systemmatrix
A = [0, 1, 0,   0;
     0, 0, a23, 0;
     0, 0, 0,   1;
     0, 0, a43, 0];
 
% Eingangsvektor
b = [0; 1/mK; 0; -1/(L*mK)];

% Ausgangsvektor
cT = [1,0,0,0];

%% Riccati-Entwurf
% Gewichtungsmatrix der Zustandsgren
Q = [1e6,0,  0,  0;
     0,  1,  0,  0;
     0,  0,  1e4,0;
     0,  0,  0,  1];

% Gewichtung der Stellgre
R = 1;

% Numerische Lsung der algebraischen Matrix-Riccati-Gleichung
% P0*A+A'*P0-P0*B*inv(R)*B'*P0+Q = 0
P0 = care(A,b,Q,R);

% Berechnung des Reglervektors
k = R^(-1)*b'*P0; 

% Vorfilter des Riccati-Reglers (vergleiche RLM Kap 3.2)
M = -(cT*(A-b*k)^(-1)*b)^(-1);  

%% Ausgabe des Reglers

disp('--------------------------------Riccati-Regler------------------------------');
disp(['Reglermatrix (k): ' num2str(k)]);
disp(['Vorfilter (M): ' num2str(M)]);
disp('Eigenwerte: ');
disp(num2str(eig(A-b*k)));
disp('----------------------------------------------------------------------------');

%% Simulation und Darstellung der Ergebnisse 

% Simuliere Modell
sim('Simulation_Verladebruecke.mdl')
% sim('Simulation_Verladebruecke_mit_Lin_Strecke.slx')

% Extrahiere Variablen
t = x.time;
x = x.signals.values;
u = u.signals.values;

% Wandle x3 und x4 von rad bzw. rad/s in grad bzw. grad/s
x(:,3:4) = x(:,3:4).*180/pi;

% y-Achsenbeschriftungen und Linienart der einzelnen Sub-Plots
label_y = {'x_K in m';'v_K in m/s';'\phi in grad'; ...
            '\omega in grad/s'}; 

% Verlufe des Zustandsvektors 
figure(1)
for ii=1:4
    subplot(2,2,ii)
    plot(t,x(:,ii),'LineWidth',2)
    xlabel('t in s')
    ylabel(label_y{ii});
end

% Verkauf der Stellgre
figure(2)
plot(t,u,'LineWidth',2)
xlabel('t in s')
ylabel('F in N')

%------------- END CODE --------------



